package org.dyndns.widerstand.proxy.grizzly;
import java.io.IOException;
import java.net.InetAddress;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.filter.ReadFilter;

public class Proxy {

    private static final Logger logger = Logger.getLogger(Proxy.class.getName());

    public Proxy(int listenPort, final String remoteServer, final int remoteServerPort) throws IOException {
        super();
        Controller controller = new Controller();
        controller.setProtocolChainInstanceHandler(new DefaultProtocolChainInstanceHandler() {

            public ProtocolChain poll() {
                ProtocolChain protocolChain = protocolChains.poll();
                if (protocolChain == null) {
                    protocolChain = new DefaultProtocolChain();
                    protocolChain.addFilter(new ReadFilter());
                    protocolChain.addFilter(new ProxyFilter(remoteServer, remoteServerPort));
                }
                return protocolChain;
            }
        });

        TCPSelectorHandler tcpSelectorHandler = new TCPSelectorHandler();
        tcpSelectorHandler.setPort(listenPort);
        tcpSelectorHandler.setInet(InetAddress.getByName("localhost"));
        controller.setSelectorHandler(tcpSelectorHandler);

        try {
            if (logger.isLoggable(Level.FINE)) {
                logger.log(Level.FINE, "Proxy starting ...");
            }
            controller.start();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                controller.stop();
                if (logger.isLoggable(Level.FINE)) {
                    logger.log(Level.FINE, "Proxy stopped.");
                }
            } catch (IOException e) {
                logger.log(Level.SEVERE, "Exception in proxy.", e);
            }
        }
    }

    public static void main(String[] args) throws Exception {
        try {
            int listenPort = Integer.parseInt(args[0]);
            String remoteServer = args[1];
            int remoteServerPort = Integer.parseInt(args[2]);
            Logger.getLogger("grizzly").setLevel(Level.OFF);
            Logger.getLogger(Proxy.class.getName()).setLevel(Level.INFO);
            new Proxy(listenPort, remoteServer, remoteServerPort);
        } catch (Exception e) {
            logger.log(Level.SEVERE, "Exception in proxy.", e);
            System.exit(1);
        }
        System.exit(0);
    }


    public static Logger getLogger() {
        return logger;
    }

}
