package org.dyndns.widerstand.proxy.grizzly;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.grizzly.Context;
import com.sun.grizzly.ProtocolFilter;
import com.sun.grizzly.Controller.Protocol;
import com.sun.grizzly.util.OutputWriter;
import com.sun.grizzly.util.WorkerThread;


public class ProxyFilter implements ProtocolFilter {

    private final static Logger logger          = Proxy.getLogger();

    private String              remoteServer;

    private int                 remoteServerPort;
    
    private static Map<SocketChannel, SocketChannel> channelMapping = new HashMap<SocketChannel, SocketChannel>();

    public ProxyFilter(String remoteServer, int remoteServerPort) {
        this.remoteServer = remoteServer;
        this.remoteServerPort = remoteServerPort;
    }

    public boolean execute(Context ctx) throws IOException {
        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, "ProxyFilter.executing ...");
        }
        ByteBuffer byteBuffer = ((WorkerThread) Thread.currentThread()).getByteBuffer();

        SelectionKey selectionKey1 = ctx.getSelectionKey();
        // get SocketChannel
        SocketChannel channel1 = (SocketChannel) selectionKey1.channel();
        SocketChannel channel2;

        if (!channelMapping.containsKey(channel1)) {
            if (logger.isLoggable(Level.FINEST)) {
                logger.log(Level.FINEST, "connecting and creating new channel mapping ...");
            }
            channel2 = SocketChannel.open();
            // blocking connect
            channel2.connect(new InetSocketAddress(remoteServer, remoteServerPort));
            channel2.configureBlocking(false);

            // remember SocketChannel (client proxy) and (proxy server) in Map
            channelMapping.put(channel1, channel2);
            // and other direction
            channelMapping.put(channel2, channel1);
        } else {
            if (logger.isLoggable(Level.FINEST)) {
                logger.log(Level.FINEST, "using existing channel mapping ...");
            }
            channel2 = channelMapping.get(channel1);
        }

        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, "writing bytes to "+channel2+"...");
        }
        byteBuffer.flip();
        // write all out
        long n = OutputWriter.flushChannel(channel2, byteBuffer);

        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, n + " bytes written.");
        }
        byteBuffer.clear();


//        // reregister reading
//        if (logger.isLoggable(Level.FINER)) {
//            logger.log(Level.FINER, "reregister read channel for reading ...");
//        }
//        ctx.getController().registerKey(selectionKey1, SelectionKey.OP_READ, Protocol.TCP);

        if (logger.isLoggable(Level.FINER)) {
            logger.log(Level.FINER, "reregister write channel for reading ...");
        }
        channel2.register(ctx.getController().getSelectorHandler(Protocol.TCP).getSelector(), SelectionKey.OP_READ);

        return true;
    }

    public boolean postExecute(Context ctx) throws IOException {
        if (ctx.getKeyRegistrationState()
                == Context.KeyRegistrationState.CANCEL){
            ctx.getController().cancelKey(ctx.getSelectionKey());
        } else if (ctx.getKeyRegistrationState()
                == Context.KeyRegistrationState.REGISTER){
            ctx.getController().registerKey(ctx.getSelectionKey());
        }
        return true;
    }
}
